@@ -7,10 +7,29 @@ import copy
 
 '''
 Compress svg figure through a variety of methods
+Written July 22, 2014
+Author: Gen Del Raye
 '''
 
 class Compress:
+    '''
+    Read in the paths in a svg file and compress selected polygons, then output the compressed
+    svg to file. The general workflow should follow:
+        comp_instance = compress(filename)                # Read svg
+        comp_instance.find_paths()                        # Extract path coordinates
+        comp_instance.linearize_paths(curve_fidelity)     # Convert paths into polygons
+        comp_instance.compress_by_x(*args, **kwargs)      # Compress polygons ( where 'x' = 'merging',
+                                                          # 'deletion', or 'simplification').
+        comp_instance.write(outputfile = outputfile)      # Output compressed file
+        comp_instance.report()                            # Report on compression stats
+        
+    Most of this functionality can also be accessed through a convenience function 'compress_by_method()'
+    See usage examples in test()
+    '''
     def __init__(self, svg_file):
+        '''
+        Read svg from path 'svg_file' (e.g. 'test/test_vector.svg')
+        '''
         self.filename, self.extension = svg_file.split('.') # Get filename and extension
         assert self.extension == 'svg', 'File must be an svg'
         self.figure_data = ET.parse(svg_file) # Parse svg data as xml
@@ -40,6 +59,13 @@ class Compress:
     def compress_by_deletion(self, criteria, threshold, search_within = None):
         '''
         Remove paths from svg that fail the given criteria
+        Inputs:
+            criteria(str) - criteria by which to accept or reject polygons. If 'bboxarea', 
+              the criteria is the area of the bounding box of a polygon, while if 'circumference',
+              the criteria is the circumference of the polygon.
+            threshold(scalar) - threshold value (e.g. of area) below which polygons are marked for
+              deletion.
+            search_within(iterable) - array of indices within which to search
         '''
         deletion_index = self.search_polygons(criteria, threshold, search_within)
         self._delete_paths(deletion_index)
@@ -52,6 +78,11 @@ class Compress:
         '''
         Find indices of polygons that fail given criteria
         Inputs:
+            criteria(string) - criteria by which to accept or reject polygons. If 'bboxarea', 
+              the criteria is the area of the bounding box of a polygon, while if 'circumference',
+              the criteria is the circumference of the polygon.
+            threshold(scalar) - threshold value (e.g. of area) below which polygons are marked for
+              deletion.
             search_within(ndarray) - array of indices within which to search
         '''
         polygons = self._coord2polygon(self.linear_coords)
@@ -101,7 +132,7 @@ class Compress:
     
     def _coord2polygon(self, coords):
         '''
-        Convert a list or array of coordinates into a polygon object
+        Convert a list or array of coordinates in coords into a polygon object
         '''
         return [vector_utils.Polygon(coord) for coord in coords]
     
@@ -109,6 +140,8 @@ class Compress:
         '''
         Simplify polygons in the svg using the Ramer-Douglas-Peucker algorithm
         Inputs:
+            epsilon(scalar) - scalar metric of how far the simplified line is allowed to 
+              stray from the original.
             search_within(ndarray) - indices of paths to simplify
         '''
         replace_coords = numpy.array(self.linear_coords)
@@ -123,7 +156,8 @@ class Compress:
     
     def overlap_list(self, poly, polyarray, search_index):
         '''
-        Return list of polygons in an array that intersect with poly
+        Return list of polygons in a polyarray that intersect with poly
+        Written to be used as port of recursive_intersect()
         '''
         overlapList = list()
         indexList = list()
@@ -135,6 +169,11 @@ class Compress:
     
     def recursive_intersect(self, poly, pArray, intersectList = [], intersectIndex = [], search_index = [], 
                             nterminate = numpy.Inf, bufferDistance = None):
+        '''
+        Find all polygons that either intersect with, or intersect with polygons that intersect with,
+        a given polygon.
+        Written to be used as part of compress_by_merging
+        '''
         if bufferDistance is not None:
             poly.buffer(bufferDistance)
         childeren, childeren_index = self.overlap_list(poly, pArray, search_index)
@@ -153,18 +192,34 @@ class Compress:
     
     def _merge(self, polygon, intersect):
         '''
-        Append polygon by cascading union
+        Merge polygons in intersect with polygon using the cascading union
         '''
         polygon.cascading_union(intersect[1:])
         
     def _hull(self, polygon, intersect):
         '''
-        Append polygon by convex hull
+        Merge polygons in intersect with polygon using the minimum convex hull
         '''
         polygon.hull(intersect[1:])
     
-    def compress_by_merging(self, search_within = None, epsilon = 1, nterminate = numpy.Inf, group_by_color = True, bufferDistance = None,
-                            operation_key = 'merge'):
+    def compress_by_merging(self, search_within = None, epsilon = 1, nterminate = numpy.Inf, group_by_color = True, 
+                            bufferDistance = None, operation_key = 'merge'):
+        '''
+        Compress svg by merging overlapping or neighboring polygons
+        Inputs:
+            search_within(iterable) - a list or 1d array of indices for which to search in self.path_nodes.
+              Optional, defaults to None.
+            epsilon(scalar) - metric of how far the simplified line is allowed to stray from the original.
+              Optional, defaults to 1
+            nterminate(scalar) - upper limit to the number of recursions that should be performed while
+              searching for overlapping or neighboring polygons. Optional, defaults to numpy.Inf
+            group_by_color(bool) - if False, merge neighboring or overlapping polygons of separate colors.
+              Optional, defaults to True.
+            bufferDistance(scalar) - limit of distance by which two polygons can be separated and still be
+              merged. Defaults to None.
+            operation_key(str) - the method by which merged polygons should be constructed. 'merge' uses
+              shapely.cascading_union, 'hull' constructs a minimum convex hull.
+        '''
         get_operation = {'merge': self._merge,
                          'hull': self._hull}
         operation = get_operation[operation_key]
@@ -182,6 +237,7 @@ class Compress:
             search_space = numpy.intersect1d(search_within, color, assume_unique = True)
             search_coords = numpy.array(self.linear_coords)[search_space]
             search_index = list(search_space)
+            # TODO: Make the polygon search more efficient by using rtrees
             polygons = self._coord2polygon(search_coords)
             while len(polygons)>0:
                 p1 = polygons.pop()
@@ -205,6 +261,10 @@ class Compress:
         
         
     def _replace_paths(self, replacement_index, replacement_str):
+        '''
+        Replace path coordinates in an svg with a string formatted in svg 
+        format (e.g. 'M 50,50 50,100 100,100 100,50 50,50 z')
+        '''
         for index, path in zip(replacement_index, replacement_str):
 #             self.path_nodes[index].set('d', 'M 50,50 50,100 100,100 100,50 50,50 z') # debug
             self.path_nodes[index].set('d', path)
@@ -230,6 +290,9 @@ class Compress:
         print self.reportString
 
     def _append_report(self, new_text):
+        '''
+        Add a new line to the report
+        '''
         self.reportString = '\n'.join((self.reportString, new_text))
     
     def _get_tag_prefix(self):
@@ -242,14 +305,42 @@ class Compress:
         return self.root.tag[:(prefix_endindex + 1)]
     
     def _get_kb(self, filename):
+        '''
+        Return size of the file 'filename' in KB.
+        '''
         return os.path.getsize(filename)/1000.0
 
 def compress_by_method(filename, compression_type, curve_fidelity = 10, outputfile = None,
                        pre_select = False, selection_tuple = ('', ''), **kwargs):
     '''
-    Convenience method for one-pass compression
-    See compress_by_merging, compress_by_deletion, and compress_by_simplification in Compress
-    for kwargs.
+    Convenience function for 1-step compression of svg files.
+    Inputs:
+        filename(str) - Path to svg file to compress, e.g. 'test_vector.svg'
+        compression_type(str) - how to carry out the compression. Options are 'delete' 
+          (remove polygons by size), 'simplify' (simplify polygons), and 'merge' (merge 
+          neighboring or overlapping polygons).
+        curve_fidelity(int) - All polygons affected by compression (e.g. those that are 
+          simplified) need to first be linearized so as to convert smooth curves into a set
+          of discrete coordinates. curve_fidelity sets the number of coordinates to use to 
+          interpolate a curve. The larger the number, the more computation the code will need 
+          to perform. Optional, defaults to 10.
+        outputfile(str) - Path to output svg. Optional, defaults to 'originalname_compressed.svg'
+        pre_select(bool) - If True, will perform the compression only on a subset of the 
+          polygons in the svg that are below a certain size threshold. Optional, defaults to False.
+        selection_tuple(tuple) - If pre_select == True, the selection tuple ('criteria', 
+          threshold) determines the criteria for selection ('bboxarea' for the area of the 
+          bounding box of a polygon, or 'circumference' for its circumference) and the threshold 
+          size. Optional, defaults to ('', '')
+        **kwargs - Depending on the compression type and the pre-selection criteria, additional 
+          key word arguments may be required. See compress_by_merging, compress_by_deletion, and 
+          compress_by_simplification in class Compress for the valid keyword arguments when 
+          compression_type = 'merge', 'delete', and 'simplify' respectively.
+    Outputs:
+        Compressed svg in the directory given by outputfile or in the code directory with the 
+          name 'originalname_compressed.svg'.
+        A report string outputted to the console will also state some general indicators about 
+          the success of the compression, the path of the outputted svg, and the initial and 
+          compressed file sizes.
     '''
     def compression_options(compress_instance, option_key, **kwargs):
         compression_options = {'merge': compress_instance.compress_by_merging,
